@@ -33,6 +33,7 @@
 
 #include "config.h"
 #include "gdk.h"
+#include "gdkwindowimpl.h"
 #include "gdkwindow.h"
 
 #include "gdkdirectfb.h"
@@ -67,6 +68,9 @@ static void gdk_window_impl_directfb_init       (GdkWindowImplDirectFB      *win
 static void gdk_window_impl_directfb_class_init (GdkWindowImplDirectFBClass *klass);
 static void gdk_window_impl_directfb_finalize   (GObject                    *object);
 
+static void gdk_window_impl_iface_init (GdkWindowImplIface *iface);
+
+
 typedef struct
 {
   GdkWindowChildChanged  changed;
@@ -88,19 +92,21 @@ gdk_window_directfb_process_all_updates (void)
 
   if (update_idle)
     g_source_remove (update_idle);
-  
+
   update_windows = NULL;
   update_idle = 0;
 
   D_DEBUG_AT( GDKDFB_Updates, "%s()\n", __FUNCTION__ );
-  
+
   g_slist_foreach (old_update_windows, (GFunc)g_object_ref, NULL);
   tmp_list = old_update_windows;
   while (tmp_list)
     {
-      GdkWindowObject       *private = GDK_WINDOW_OBJECT( tmp_list->data );
-      GdkWindowImplDirectFB *wimpl   = GDK_WINDOW_IMPL_DIRECTFB( private->impl );
-      
+      GdkWindowObject *private = GDK_WINDOW_OBJECT( tmp_list->data );
+#ifdef DIRECT_ENABLE_DEBUG
+      GdkWindowImplDirectFB *wimpl = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
+#endif
+
       if (private->update_freeze_count)
         {
           D_DEBUG_AT( GDKDFB_Updates, "  -> %p frozen [%4d,%4d-%4dx%4d] (%d boxes)\n",
@@ -115,7 +121,7 @@ gdk_window_directfb_process_all_updates (void)
                       wimpl->flips.num_regions );
           gdk_window_process_updates(tmp_list->data,TRUE);
         }
-      
+
       g_object_unref (tmp_list->data);
       tmp_list = tmp_list->next;
     }
@@ -159,7 +165,7 @@ static gboolean
 gdk_window_update_idle (gpointer data)
 {
   gdk_window_directfb_process_all_updates ();
-  
+
   return FALSE;
 }
 
@@ -168,7 +174,7 @@ gdk_window_schedule_update (GdkWindow *window)
 {
   D_DEBUG_AT( GDKDFB_Updates, "%s( %p ) <- freeze count %d\n", __FUNCTION__, window,
               window ? GDK_WINDOW_OBJECT (window)->update_freeze_count : -1 );
-  
+
   if (window && GDK_WINDOW_OBJECT (window)->update_freeze_count)
     return;
 
@@ -186,11 +192,8 @@ static GdkWindow *gdk_directfb_window_containing_pointer = NULL;
 static GdkWindow *gdk_directfb_focused_window            = NULL;
 static gpointer   parent_class                           = NULL;
 GdkWindow * _gdk_parent_root = NULL;
-static void
-gdk_window_impl_directfb_paintable_init (GdkPaintableIface *iface);
-
-
 
+static void gdk_window_impl_directfb_paintable_init (GdkPaintableIface *iface);
 
 
 GType
@@ -213,20 +216,30 @@ gdk_window_impl_directfb_get_type (void)
           (GInstanceInitFunc) gdk_window_impl_directfb_init,
         };
 
-    static const GInterfaceInfo paintable_info =
-      {
-    (GInterfaceInitFunc) gdk_window_impl_directfb_paintable_init,
-    NULL,
-    NULL
-      };
+      static const GInterfaceInfo paintable_info =
+        {
+          (GInterfaceInitFunc) gdk_window_impl_directfb_paintable_init,
+          NULL,
+          NULL
+        };
+
+      static const GInterfaceInfo window_impl_info =
+        {
+          (GInterfaceInitFunc) gdk_window_impl_iface_init,
+          NULL,
+          NULL
+        };
 
       object_type = g_type_register_static (GDK_TYPE_DRAWABLE_IMPL_DIRECTFB,
                                             "GdkWindowImplDirectFB",
                                             &object_info, 0);
-       g_type_add_interface_static (object_type,
-                   GDK_TYPE_PAINTABLE,
-                   &paintable_info);
+      g_type_add_interface_static (object_type,
+                                   GDK_TYPE_PAINTABLE,
+                                   &paintable_info);
 
+      g_type_add_interface_static (object_type,
+                                   GDK_TYPE_WINDOW_IMPL,
+                                   &window_impl_info);
     }
 
   return object_type;
@@ -371,7 +384,7 @@ create_directfb_window (GdkWindowImplDirectFB *impl,
   if ((desc->flags & DWDESC_CAPS) && (desc->caps & DWCAPS_INPUTONLY))
   {
     impl->drawable.surface = NULL;
-  } else 
+  } else
     window->GetSurface (window, &impl->drawable.surface);
 
   if (window_options)
@@ -401,11 +414,11 @@ _gdk_windowing_window_init (void)
 
   g_assert (_gdk_parent_root == NULL);
 
-  _gdk_display->layer->GetConfiguration( 
-	_gdk_display->layer, &dlc );
+  _gdk_display->layer->GetConfiguration (_gdk_display->layer, &dlc);
 
   _gdk_parent_root = g_object_new (GDK_TYPE_WINDOW, NULL);
   private = GDK_WINDOW_OBJECT (_gdk_parent_root);
+  private->impl = g_object_new (_gdk_window_impl_get_type (), NULL);
   impl = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
 
   private->window_type = GDK_WINDOW_ROOT;
@@ -439,7 +452,7 @@ _gdk_windowing_window_init (void)
 	g_assert(impl->window != NULL);
     g_assert(impl->drawable.surface != NULL );
   }
-  impl->drawable.surface->GetPixelFormat(impl->drawable.surface,&impl->drawable.format); 
+  impl->drawable.surface->GetPixelFormat(impl->drawable.surface,&impl->drawable.format);
   private->depth = DFB_BITS_PER_PIXEL(impl->drawable.format);
   /*
 	Now we can set up the system colormap
@@ -475,6 +488,7 @@ gdk_directfb_window_new (GdkWindow              *parent,
 
   window = g_object_new (GDK_TYPE_WINDOW, NULL);
   private = GDK_WINDOW_OBJECT (window);
+  private->impl = g_object_new (_gdk_window_impl_get_type (), NULL);
 
   parent_private = GDK_WINDOW_OBJECT (parent);
   parent_impl = GDK_WINDOW_IMPL_DIRECTFB (parent_private->impl);
@@ -647,8 +661,8 @@ gdk_directfb_window_new (GdkWindow              *parent,
 
 GdkWindow *
 _gdk_window_new (GdkWindow     *parent,
-                GdkWindowAttr *attributes,
-                gint           attributes_mask)
+                 GdkWindowAttr *attributes,
+                 gint           attributes_mask)
 {
   g_return_val_if_fail (attributes != NULL, NULL);
 
@@ -660,7 +674,7 @@ _gdk_window_new (GdkWindow     *parent,
 void
 _gdk_windowing_window_destroy_foreign (GdkWindow *window)
 {
-  /* It's somebody else's window, but in our heirarchy,
+  /* It's somebody else's window, but in our hierarchy,
    * so reparent it to the root window, and then send
    * it a delete event, as if we were a WM
    */
@@ -685,9 +699,8 @@ _gdk_windowing_window_destroy (GdkWindow *window,
   impl = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
 
   _gdk_selection_window_destroyed (window);
-#if (DIRECTFB_MAJOR_VERSION >= 1)
   gdk_directfb_event_windows_remove (window);
-#endif
+
   if (window == _gdk_directfb_pointer_grab_window)
     gdk_pointer_ungrab (GDK_CURRENT_TIME);
   if (window == _gdk_directfb_keyboard_grab_window)
@@ -816,11 +829,11 @@ gdk_window_set_accept_focus (GdkWindow *window,
   GdkWindowObject *private;
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
-                                                                                                                      
+
   private = (GdkWindowObject *)window;
-                                                                                                                      
+
   accept_focus = accept_focus != FALSE;
-                                                                                                                      
+
   if (private->accept_focus != accept_focus)
     private->accept_focus = accept_focus;
 
@@ -833,11 +846,11 @@ gdk_window_set_focus_on_map (GdkWindow *window,
   GdkWindowObject *private;
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
-                                                                                                                      
+
   private = (GdkWindowObject *)window;
-                                                                                                                      
+
   focus_on_map = focus_on_map != FALSE;
-                                                                                                                      
+
   if (private->focus_on_map != focus_on_map)
     private->focus_on_map = focus_on_map;
 }
@@ -1230,28 +1243,19 @@ show_window_internal (GdkWindow *window,
     }
 }
 
-void
-gdk_window_show_unraised (GdkWindow *window)
+static void
+gdk_directfb_window_show (GdkWindow *window,
+                          gboolean   raise)
 {
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   D_DEBUG_AT( GDKDFB_Window, "%s( %p )\n", __FUNCTION__, window );
 
-  show_window_internal (window, FALSE);
+  show_window_internal (window, raise);
 }
 
-void
-gdk_window_show (GdkWindow *window)
-{
-  g_return_if_fail (GDK_IS_WINDOW (window));
-
-  D_DEBUG_AT( GDKDFB_Window, "%s( %p )\n", __FUNCTION__, window );
-
-  show_window_internal (window, TRUE);
-}
-
-void
-gdk_window_hide (GdkWindow *window)
+static void
+gdk_directfb_window_hide (GdkWindow *window)
 {
   GdkWindowObject       *private;
   GdkWindowImplDirectFB *impl;
@@ -1276,11 +1280,11 @@ gdk_window_hide (GdkWindow *window)
 
       if (!private->input_only && private->parent)
         {
-          _gdk_windowing_window_clear_area (GDK_WINDOW (private->parent),
-                                            private->x,
-                                            private->y,
-                                            impl->drawable.width,
-                                            impl->drawable.height);
+          gdk_window_clear_area (GDK_WINDOW (private->parent),
+                                 private->x,
+                                 private->y,
+                                 impl->drawable.width,
+                                 impl->drawable.height);
         }
 
       event_win = gdk_directfb_other_event_window (window, GDK_UNMAP);
@@ -1299,8 +1303,8 @@ gdk_window_hide (GdkWindow *window)
     }
 }
 
-void
-gdk_window_withdraw (GdkWindow *window)
+static void
+gdk_directfb_window_withdraw (GdkWindow *window)
 {
   g_return_if_fail (GDK_IS_WINDOW (window));
 
@@ -1308,80 +1312,6 @@ gdk_window_withdraw (GdkWindow *window)
   gdk_window_hide (window);
 }
 
-void
-gdk_window_move (GdkWindow *window,
-                 gint       x,
-                 gint       y)
-{
-  GdkWindowObject       *private;
-  GdkWindowImplDirectFB *impl;
-
-  g_return_if_fail (GDK_IS_WINDOW (window));
-
-  private = GDK_WINDOW_OBJECT (window);
-  impl    = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
-
-  if (impl->window)
-    {
-  	  private->x = x;
-  	  private->y = y;
-      impl->window->MoveTo (impl->window, x, y);
-    }
-  else
-    {
-	  gint width=impl->drawable.width;
-	  gint height=impl->drawable.height;
-      GdkRectangle  old =
-      { private->x, private->y,width,height };
-
-      _gdk_directfb_move_resize_child (window, x, y, width, height);
-      _gdk_directfb_calc_abs (window);
-
-      if (GDK_WINDOW_IS_MAPPED (private))
-        {
-          GdkWindow    *mousewin;
-          GdkRectangle  new = { x, y, width, height };
-
-          gdk_rectangle_union (&new, &old, &new);
-          gdk_window_invalidate_rect (GDK_WINDOW (private->parent), &new,TRUE);
-
-          /* The window the pointer is in might have changed */
-          mousewin = gdk_window_at_pointer (NULL, NULL);
-          gdk_directfb_window_send_crossing_events (NULL, mousewin,
-                                                    GDK_CROSSING_NORMAL);
-        }
-    }
-}
-
-void
-gdk_window_resize (GdkWindow *window,
-                   gint       width,
-                   gint       height)
-{
-  GdkWindowObject *private;
-  gint             x, y;
-
-  g_return_if_fail (GDK_IS_WINDOW (window));
-
-  private = GDK_WINDOW_OBJECT (window);
-
-  x = private->x;
-  y = private->y;
-
-  if (private->parent && (private->parent->window_type != GDK_WINDOW_CHILD))
-    {
-      GdkWindowChildHandlerData *data;
-
-      data = g_object_get_data (G_OBJECT (private->parent),
-				"gdk-window-child-handler");
-
-      if (data)
-	(*data->get_pos) (window, &x, &y, data->user_data);
-    }
-
-  gdk_window_move_resize (window, x, y, width, height);
-}
-
 void
 _gdk_directfb_move_resize_child (GdkWindow *window,
                                  gint       x,
@@ -1407,15 +1337,17 @@ _gdk_directfb_move_resize_child (GdkWindow *window,
 
   if (!private->input_only)
     {
-    if (impl->drawable.surface) {
-      GdkDrawableImplDirectFB *dimpl = GDK_DRAWABLE_IMPL_DIRECTFB (private->impl);
-      if(dimpl->cairo_surface) {
-        cairo_surface_destroy(dimpl->cairo_surface);
-        dimpl->cairo_surface= NULL;
-      }
-    impl->drawable.surface->Release (impl->drawable.surface);
-    impl->drawable.surface = NULL;
-  }
+      if (impl->drawable.surface)
+        {
+          if (impl->drawable.cairo_surface)
+            {
+              cairo_surface_destroy (impl->drawable.cairo_surface);
+              impl->drawable.cairo_surface = NULL;
+            }
+
+          impl->drawable.surface->Release (impl->drawable.surface);
+          impl->drawable.surface = NULL;
+        }
 
       parent_impl = GDK_WINDOW_IMPL_DIRECTFB (GDK_WINDOW_OBJECT (private->parent)->impl);
 
@@ -1432,19 +1364,66 @@ _gdk_directfb_move_resize_child (GdkWindow *window,
   for (list = private->children; list; list = list->next)
     {
       private = GDK_WINDOW_OBJECT (list->data);
-  	  impl  = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
+      impl  = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
+
       _gdk_directfb_move_resize_child (list->data,
                                        private->x, private->y,
                                        impl->drawable.width, impl->drawable.height);
     }
 }
 
-void
-gdk_window_move_resize (GdkWindow *window,
-                        gint       x,
-                        gint       y,
-                        gint       width,
-                        gint       height)
+static  void
+gdk_directfb_window_move (GdkWindow *window,
+                          gint       x,
+                          gint       y)
+{
+  GdkWindowObject       *private;
+  GdkWindowImplDirectFB *impl;
+
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  private = GDK_WINDOW_OBJECT (window);
+  impl    = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
+
+  if (impl->window)
+    {
+      private->x = x;
+      private->y = y;
+      impl->window->MoveTo (impl->window, x, y);
+    }
+  else
+    {
+         gint width=impl->drawable.width;
+         gint height=impl->drawable.height;
+      GdkRectangle  old =
+      { private->x, private->y,width,height };
+
+      _gdk_directfb_move_resize_child (window, x, y, width, height);
+      _gdk_directfb_calc_abs (window);
+
+      if (GDK_WINDOW_IS_MAPPED (private))
+        {
+          GdkWindow    *mousewin;
+          GdkRectangle  new = { x, y, width, height };
+
+          gdk_rectangle_union (&new, &old, &new);
+          gdk_window_invalidate_rect (GDK_WINDOW (private->parent), &new,TRUE);
+
+          /* The window the pointer is in might have changed */
+          mousewin = gdk_window_at_pointer (NULL, NULL);
+          gdk_directfb_window_send_crossing_events (NULL, mousewin,
+                                                    GDK_CROSSING_NORMAL);
+        }
+    }
+}
+
+static void
+gdk_directfb_window_move_resize (GdkWindow *window,
+                                 gboolean   with_move,
+                                 gint       x,
+                                 gint       y,
+                                 gint       width,
+                                 gint       height)
 {
   GdkWindowObject       *private;
   GdkWindowImplDirectFB *impl;
@@ -1454,8 +1433,14 @@ gdk_window_move_resize (GdkWindow *window,
   private = GDK_WINDOW_OBJECT (window);
   impl = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
 
+  if (with_move && (width < 0 && height < 0))
+    {
+      gdk_directfb_window_move (window, x, y);
+      return;
+    }
+
   if (width < 1)
-     width = 1;
+    width = 1;
   if (height < 1)
     height = 1;
 
@@ -1478,26 +1463,39 @@ gdk_window_move_resize (GdkWindow *window,
 
   if (impl->drawable.width == width  &&  impl->drawable.height == height)
     {
-      gdk_window_move (window, x, y);
+      if (with_move)
+        gdk_directfb_window_move (window, x, y);
     }
   else if (impl->window)
     {
-  	  private->x = x;
-  	  private->y = y;
-      impl->window->MoveTo (impl->window, x, y);
+      private->x = x;
+      private->y = y;
+      impl->drawable.width = width;
+      impl->drawable.height = height;
+
+      if (with_move)
+        impl->window->MoveTo (impl->window, x, y);
       impl->window->Resize (impl->window, width, height);
     }
   else
     {
-      GdkRectangle  old =
-      { private->x, private->y, impl->drawable.width, impl->drawable.height };
-      _gdk_directfb_move_resize_child (window, x, y, width, height);
+      GdkRectangle old = { private->x, private->y,
+                           impl->drawable.width, impl->drawable.height };
+      GdkRectangle new = { x, y, width, height };
+
+      if (! with_move)
+        {
+          new.x = private->x;
+          new.y = private->y;
+        }
+
+      _gdk_directfb_move_resize_child (window,
+                                       new.x, new.y, new.width, new.height);
       _gdk_directfb_calc_abs (window);
 
       if (GDK_WINDOW_IS_MAPPED (private))
         {
-          GdkWindow    *mousewin;
-          GdkRectangle  new = { x, y, width, height };
+          GdkWindow *mousewin;
 
           gdk_rectangle_union (&new, &old, &new);
           gdk_window_invalidate_rect (GDK_WINDOW (private->parent), &new,TRUE);
@@ -1510,11 +1508,11 @@ gdk_window_move_resize (GdkWindow *window,
     }
 }
 
-void
-_gdk_window_reparent (GdkWindow *window,
-                     GdkWindow *new_parent,
-                     gint       x,
-                     gint       y)
+static gboolean
+gdk_directfb_window_reparent (GdkWindow *window,
+                              GdkWindow *new_parent,
+                              gint       x,
+                              gint       y)
 {
   GdkWindowObject *window_private;
   GdkWindowObject *parent_private;
@@ -1523,10 +1521,10 @@ _gdk_window_reparent (GdkWindow *window,
   GdkWindowImplDirectFB *parent_impl;
   GdkVisual             *visual;
 
-  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
 
   if (GDK_WINDOW_DESTROYED (window))
-    return;
+    return FALSE;
 
   if (!new_parent)
     new_parent = _gdk_parent_root;
@@ -1539,7 +1537,7 @@ _gdk_window_reparent (GdkWindow *window,
 
   /* already parented */
   if( window_private->parent == (GdkWindowObject *)new_parent )
-          return;
+          return FALSE;
 
   window_private->parent = (GdkWindowObject *) new_parent;
 
@@ -1558,7 +1556,7 @@ _gdk_window_reparent (GdkWindow *window,
         impl->drawable.surface = NULL;
     }
 
-    if( impl->window != NULL ) { 
+    if( impl->window != NULL ) {
         gdk_directfb_window_id_table_remove (impl->dfb_id);
     	impl->window->SetOpacity (impl->window,0);
    		impl->window->Close(impl->window);
@@ -1593,7 +1591,7 @@ _gdk_window_reparent (GdkWindow *window,
         {
 		  g_assert(0);
           _gdk_window_destroy (window, FALSE);
-          return;
+          return FALSE;
         }
         /* we hold a reference count on ourselves */
         g_object_ref (window);
@@ -1609,14 +1607,17 @@ _gdk_window_reparent (GdkWindow *window,
                           &rect,
                           &impl->drawable.surface);
    }
+
+   return TRUE;
 }
 
-void
-_gdk_windowing_window_clear_area (GdkWindow *window,
-                                  gint       x,
-                                  gint       y,
-                                  gint       width,
-                                  gint       height)
+static void
+gdk_directfb_window_clear_area (GdkWindow *window,
+                                gint       x,
+                                gint       y,
+                                gint       width,
+                                gint       height,
+                                gboolean   send_expose)
 {
   GdkWindowObject         *private;
   GdkDrawableImplDirectFB *impl;
@@ -1634,7 +1635,7 @@ _gdk_windowing_window_clear_area (GdkWindow *window,
     D_DEBUG_AT( GDKDFB_Window, "  -> DESTROYED!\n" );
     return;
   }
-  
+
   private = GDK_WINDOW_OBJECT (window);
 
   impl = GDK_DRAWABLE_IMPL_DIRECTFB (private->impl);
@@ -1642,9 +1643,9 @@ _gdk_windowing_window_clear_area (GdkWindow *window,
   /**
 	Follow XClearArea definition for zero height width
   **/
-  if( width == 0 )  
+  if( width == 0 )
 		width = impl->width-x;
-  if( height == 0 )  
+  if( height == 0 )
 		height = impl->height-y;
 
   bg_pixmap = private->bg_pixmap;
@@ -1678,7 +1679,7 @@ _gdk_windowing_window_clear_area (GdkWindow *window,
                                    GDK_GC_FILL | GDK_GC_TILE |
                                    GDK_GC_TS_X_ORIGIN | GDK_GC_TS_Y_ORIGIN);
     }
-  else 
+  else
     {
       /* GDK_PARENT_RELATIVE_BG, but no pixmap,
          get the color from the parent window. */
@@ -1700,51 +1701,8 @@ _gdk_windowing_window_clear_area (GdkWindow *window,
     g_object_unref (gc);
 }
 
-void
-_gdk_windowing_window_clear_area_e (GdkWindow *window,
-                                    gint       x,
-                                    gint       y,
-                                    gint       width,
-                                    gint       height)
-{
-  GdkRectangle  rect;
-  GdkWindowObject       *private;
-  GdkWindowImplDirectFB *impl;
-
-  D_DEBUG_AT( GDKDFB_Window, "%s( %p, %4d,%4d-%4dx%4d )\n", __FUNCTION__, window, x, y, width, height );
-
-  g_return_if_fail (GDK_IS_WINDOW (window));
-
-  private = GDK_WINDOW_OBJECT (window);
-  impl    = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
-
-  /**
-	Follow XClearArea definition for zero height width
-  **/
-  if( width == 0 )  
-		width = impl->drawable.width-x;
-  if( height == 0 )  
-		height = impl->drawable.height-y;
-
-  rect.x = x;
-  rect.y = y;
-  rect.width = width;
-  rect.height = height;
-
-  g_return_if_fail (GDK_IS_WINDOW (window));
-
-  if (GDK_WINDOW_DESTROYED (window)) {
-    D_DEBUG_AT( GDKDFB_Window, "  -> DESTROYED!\n" );
-    return;
-  }
-
-  _gdk_windowing_window_clear_area (window, x, y, width, height);
-
-  gdk_window_invalidate_rect (window, &rect, TRUE);
-}
-
-void
-gdk_window_raise (GdkWindow *window)
+static void
+gdk_window_directfb_raise (GdkWindow *window)
 {
   GdkWindowImplDirectFB *impl;
 
@@ -1774,8 +1732,8 @@ gdk_window_raise (GdkWindow *window)
     }
 }
 
-void
-gdk_window_lower (GdkWindow *window)
+static void
+gdk_window_directfb_lower (GdkWindow *window)
 {
   GdkWindowImplDirectFB *impl;
 
@@ -1875,9 +1833,9 @@ gdk_window_set_role (GdkWindow   *window,
  * Since: 2.12
  *
  **/
-void          
+void
 gdk_window_set_startup_id (GdkWindow   *window,
-		     const gchar *startup_id)
+                           const gchar *startup_id)
 {
 }
 
@@ -1907,9 +1865,9 @@ gdk_window_set_transient_for (GdkWindow *window,
     root->children = g_list_insert (root->children, window, i);
 }
 
-void
-gdk_window_set_background (GdkWindow *window,
-                           const GdkColor  *color)
+static void
+gdk_directfb_window_set_background (GdkWindow *window,
+                                    const GdkColor  *color)
 {
   GdkWindowObject *private;
 
@@ -1930,10 +1888,10 @@ gdk_window_set_background (GdkWindow *window,
   private->bg_pixmap = NULL;
 }
 
-void
-gdk_window_set_back_pixmap (GdkWindow *window,
-                            GdkPixmap *pixmap,
-                            gint       parent_relative)
+static void
+gdk_directfb_window_set_back_pixmap (GdkWindow *window,
+                                     GdkPixmap *pixmap,
+                                     gboolean   parent_relative)
 {
   GdkWindowObject *private;
   GdkPixmap       *old_pixmap;
@@ -1972,9 +1930,9 @@ gdk_window_set_back_pixmap (GdkWindow *window,
     }
 }
 
-void
-gdk_window_set_cursor (GdkWindow *window,
-                       GdkCursor *cursor)
+static void
+gdk_directfb_window_set_cursor (GdkWindow *window,
+                                GdkCursor *cursor)
 {
   GdkWindowImplDirectFB *impl;
   GdkCursor             *old_cursor;
@@ -2011,13 +1969,13 @@ gdk_window_set_cursor (GdkWindow *window,
     gdk_cursor_unref (old_cursor);
 }
 
-void
-gdk_window_get_geometry (GdkWindow *window,
-                         gint      *x,
-                         gint      *y,
-                         gint      *width,
-                         gint      *height,
-                         gint      *depth)
+static void
+gdk_directfb_window_get_geometry (GdkWindow *window,
+                                  gint      *x,
+                                  gint      *y,
+                                  gint      *width,
+                                  gint      *height,
+                                  gint      *depth)
 {
   GdkWindowObject         *private;
   GdkDrawableImplDirectFB *impl;
@@ -2078,10 +2036,10 @@ _gdk_directfb_calc_abs (GdkWindow *window)
     }
 }
 
-gboolean
-gdk_window_get_origin (GdkWindow *window,
-                       gint      *x,
-                       gint      *y)
+static gboolean
+gdk_directfb_window_get_origin (GdkWindow *window,
+                                gint      *x,
+                                gint      *y)
 {
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
 
@@ -2215,8 +2173,8 @@ _gdk_windowing_window_get_pointer (display,
 
 }
 
-GdkEventMask
-gdk_window_get_events (GdkWindow *window)
+static GdkEventMask
+gdk_directfb_window_get_events (GdkWindow *window)
 {
   g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
 
@@ -2226,9 +2184,9 @@ gdk_window_get_events (GdkWindow *window)
     return GDK_WINDOW_OBJECT (window)->event_mask;
 }
 
-void
-gdk_window_set_events (GdkWindow    *window,
-                       GdkEventMask  event_mask)
+static void
+gdk_directfb_window_set_events (GdkWindow    *window,
+                                GdkEventMask  event_mask)
 {
   g_return_if_fail (GDK_IS_WINDOW (window));
 
@@ -2240,27 +2198,27 @@ gdk_window_set_events (GdkWindow    *window,
   GDK_WINDOW_OBJECT (window)->event_mask = event_mask;
 }
 
-void
-gdk_window_shape_combine_mask (GdkWindow *window,
-                               GdkBitmap *mask,
-                               gint       x,
-                               gint       y)
+static void
+gdk_directfb_window_shape_combine_mask (GdkWindow *window,
+                                        GdkBitmap *mask,
+                                        gint       x,
+                                        gint       y)
 {
 }
 
 void
 gdk_window_input_shape_combine_mask (GdkWindow *window,
-                                    GdkBitmap *mask,
-                                    gint       x,
-                                    gint       y)
+                                     GdkBitmap *mask,
+                                     gint       x,
+                                     gint       y)
 {
 }
 
-void
-gdk_window_shape_combine_region (GdkWindow       *window,
-                                 const GdkRegion *shape_region,
-                                 gint             offset_x,
-                                 gint             offset_y)
+static void
+gdk_directfb_window_shape_combine_region (GdkWindow       *window,
+                                          const GdkRegion *shape_region,
+                                          gint             offset_x,
+                                          gint             offset_y)
 {
 }
 
@@ -2458,7 +2416,7 @@ GdkWindowTypeHint
 gdk_window_get_type_hint (GdkWindow *window)
 {
   g_return_val_if_fail (GDK_IS_WINDOW (window), GDK_WINDOW_TYPE_HINT_NORMAL);
-  
+
   if (GDK_WINDOW_DESTROYED (window))
     return GDK_WINDOW_TYPE_HINT_NORMAL;
 
@@ -2517,7 +2475,7 @@ gdk_window_set_group (GdkWindow *window,
 GdkWindow * gdk_window_get_group (GdkWindow *window)
 {
  g_warning(" DirectFb get_group groups not supported \n");
- return window;	
+ return window;
 }
 
 void
@@ -2584,13 +2542,13 @@ gdk_window_set_functions (GdkWindow     *window,
   g_message("unimplemented %s", __FUNCTION__);
 }
 
-void
-gdk_window_set_child_shapes (GdkWindow *window)
+static void
+gdk_directfb_window_set_child_shapes (GdkWindow *window)
 {
 }
 
-void
-gdk_window_merge_child_shapes (GdkWindow *window)
+static void
+gdk_directfb_window_merge_child_shapes (GdkWindow *window)
 {
 }
 
@@ -2604,9 +2562,9 @@ gdk_window_merge_child_input_shapes (GdkWindow *window)
 {
 }
 
-gboolean
-gdk_window_set_static_gravities (GdkWindow *window,
-                                 gboolean   use_static)
+static gboolean
+gdk_directfb_window_set_static_gravities (GdkWindow *window,
+                                          gboolean   use_static)
 {
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
 
@@ -2693,9 +2651,8 @@ gdk_window_get_frame_extents (GdkWindow    *window,
  * Given a directfb window and a subsurface of that window
  * create a gdkwindow child wrapper
  */
-#if (DIRECTFB_MAJOR_VERSION >= 1)
 GdkWindow *gdk_directfb_create_child_window(GdkWindow *parent,
-                                IDirectFBSurface *subsurface)
+                                            IDirectFBSurface *subsurface)
 {
   GdkWindow             *window;
   GdkWindowObject       *private;
@@ -2708,6 +2665,7 @@ GdkWindow *gdk_directfb_create_child_window(GdkWindow *parent,
 
   window = g_object_new (GDK_TYPE_WINDOW, NULL);
   private = GDK_WINDOW_OBJECT (window);
+  private->impl = g_object_new (_gdk_window_impl_get_type (), NULL);
   parent_private = GDK_WINDOW_OBJECT (parent);
   parent_impl = GDK_WINDOW_IMPL_DIRECTFB (parent_private->impl);
   private->parent = parent_private;
@@ -2739,7 +2697,6 @@ GdkWindow *gdk_directfb_create_child_window(GdkWindow *parent,
   return window;
 
 }
-#endif
 
 /*
  * The wrapping is not perfect since directfb does not give full access
@@ -2787,6 +2744,7 @@ gdk_window_foreign_new_for_display (GdkDisplay* display,GdkNativeWindow anid)
     /* we hold a reference count on ourselves */
     g_object_ref (window);
     private = GDK_WINDOW_OBJECT (window);
+    private->impl = g_object_new (_gdk_window_impl_get_type (), NULL);
     private->parent = parent_private;
     private->window_type = GDK_WINDOW_TOPLEVEL;
     impl = GDK_WINDOW_IMPL_DIRECTFB (private->impl);
@@ -2809,8 +2767,8 @@ gdk_window_foreign_new_for_display (GdkDisplay* display,GdkNativeWindow anid)
      */
     _gdk_directfb_calc_abs (window);
 
-    /* We default to all events least surprise to the user 
-     * minus the poll for motion events 
+    /* We default to all events least surprise to the user
+     * minus the poll for motion events
      */
     gdk_window_set_events (window, (GDK_ALL_EVENTS_MASK ^ GDK_POINTER_MOTION_HINT_MASK));
 
@@ -2835,7 +2793,7 @@ gdk_window_foreign_new_for_display (GdkDisplay* display,GdkNativeWindow anid)
     if (parent_private)
         parent_private->children = g_list_prepend (parent_private->children,
                                                window);
-    impl->dfb_id = (DFBWindowID)anid; 
+    impl->dfb_id = (DFBWindowID)anid;
     gdk_directfb_window_id_table_insert (impl->dfb_id, window);
     gdk_directfb_event_windows_add (window);
 
@@ -2897,7 +2855,7 @@ gdk_window_set_keep_above (GdkWindow *window, gboolean setting)
   g_warning ("gdk_window_set_keep_above() not implemented.\n");
 	first_call=FALSE;
   }
-	
+
 }
 
 void
@@ -2909,7 +2867,7 @@ gdk_window_set_keep_below (GdkWindow *window, gboolean setting)
   g_warning ("gdk_window_set_keep_below() not implemented.\n");
   first_call=FALSE;
   }
-  
+
 }
 
 void
@@ -2969,7 +2927,7 @@ gdk_window_impl_directfb_invalidate_maybe_recurse (GdkPaintable    *paintable,
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
-  
+
   if (private->input_only || !GDK_WINDOW_IS_MAPPED (window))
     return;
 
@@ -2981,22 +2939,22 @@ gdk_window_impl_directfb_invalidate_maybe_recurse (GdkPaintable    *paintable,
     {
       GdkWindowObject         *child = tmp_list->data;
       GdkDrawableImplDirectFB *cimpl = (GdkDrawableImplDirectFB *) child->impl;
-      
+
       if (!child->input_only)
         {
           GdkRegion child_region;
-          
+
           temp_region_init_rectangle_vals( &child_region, child->x, child->y, cimpl->width, cimpl->height );
-          
+
           /* remove child area from the invalid area of the parent */
           if (GDK_WINDOW_IS_MAPPED (child) && !child->shaped)
             gdk_region_subtract (&visible_region, &child_region);
-          
+
           if (child_func && (*child_func) ((GdkWindow *)child, user_data))
             {
               gdk_region_intersect (&child_region, region);
               gdk_region_offset (&child_region, - child->x, - child->y);
-              
+
               gdk_window_invalidate_maybe_recurse ((GdkWindow *)child,
                                                    &child_region, child_func, user_data);
             }
@@ -3006,10 +2964,10 @@ gdk_window_impl_directfb_invalidate_maybe_recurse (GdkPaintable    *paintable,
 
       tmp_list = tmp_list->next;
     }
-  
+
   if (!gdk_region_empty (&visible_region))
     {
-      
+
       if (private->update_area)
         {
           gdk_region_union (private->update_area, &visible_region);
@@ -3053,7 +3011,7 @@ gdk_window_impl_directfb_process_updates (GdkPaintable *paintable,
 
   update_area = private->update_area;
   private->update_area = NULL;
-      
+
   D_DEBUG_AT( GDKDFB_Paintable, "  -> update area %4d,%4d-%4dx%4d\n",
               GDKDFB_RECTANGLE_VALS_FROM_BOX( &update_area->extents ) );
 
@@ -3061,15 +3019,15 @@ gdk_window_impl_directfb_process_updates (GdkPaintable *paintable,
     {
       GdkRegion *expose_region = update_area;
       GdkRegion  window_region;
-          
+
       temp_region_init_rectangle_vals( &window_region, 0, 0, impl->width, impl->height );
       gdk_region_intersect( expose_region, &window_region );
       temp_region_deinit (&window_region);
-          
+
       if (!gdk_region_empty (expose_region) && (private->event_mask & GDK_EXPOSURE_MASK))
         {
           GdkEvent event;
-              
+
           event.expose.type = GDK_EXPOSE;
           event.expose.window = g_object_ref (window);
           event.expose.send_event = FALSE;
@@ -3077,7 +3035,7 @@ gdk_window_impl_directfb_process_updates (GdkPaintable *paintable,
           event.expose.region = expose_region;
           gdk_region_get_clipbox (expose_region, &event.expose.area);
           (*_gdk_event_func) (&event, _gdk_event_data);
-              
+
           g_object_unref (window);
         }
 
@@ -3113,7 +3071,7 @@ gdk_window_impl_directfb_begin_paint_region (GdkPaintable    *paintable,
     {
       /* ...we're already painting on it! */
       g_assert( impl->paint_depth > 0 );
-    
+
       D_DEBUG_AT( GDKDFB_Window, "  -> painted  %4d,%4d-%4dx%4d (%ld boxes)\n",
                   DFB_RECTANGLE_VALS_FROM_REGION( &impl->paint_region.extents ), impl->paint_region.numRects );
 
@@ -3153,11 +3111,11 @@ gdk_window_impl_directfb_begin_paint_region (GdkPaintable    *paintable,
 
       D_DEBUG_AT( GDKDFB_Window, "  -> [%2d] %4d,%4d-%4dx%4d\n", i, GDKDFB_RECTANGLE_VALS_FROM_BOX( box ) );
 
-      _gdk_windowing_window_clear_area (GDK_WINDOW(wimpl->gdkWindow),
-                                        box->x1,
-                                        box->y1,
-                                        box->x2 - box->x1,
-                                        box->y2 - box->y1);
+      gdk_window_clear_area (GDK_WINDOW(wimpl->gdkWindow),
+                             box->x1,
+                             box->y1,
+                             box->x2 - box->x1,
+                             box->y2 - box->y1);
     }
 }
 
@@ -3200,34 +3158,34 @@ gdk_window_impl_directfb_end_paint (GdkPaintable *paintable)
   if (impl->paint_depth == 0)
     {
       impl->buffered = FALSE;
-  
+
       temp_region_deinit( &impl->clip_region );
-  
+
       if (impl->paint_region.numRects)
         {
           GdkWindow *window = GDK_WINDOW( impl->wrapper );
-  
+
           if (GDK_IS_WINDOW(window))
             {
               GdkWindowObject *top = GDK_WINDOW_OBJECT( gdk_window_get_toplevel( window ) );
-  
+
               if (top)
                 {
                   DFBRegion              reg;
                   GdkWindowImplDirectFB *wimpl = GDK_WINDOW_IMPL_DIRECTFB (top->impl);
-  
+
                   reg.x1 = impl->abs_x - top->x + impl->paint_region.extents.x1;
                   reg.y1 = impl->abs_y - top->y + impl->paint_region.extents.y1;
                   reg.x2 = impl->abs_x - top->x + impl->paint_region.extents.x2 - 1;
                   reg.y2 = impl->abs_y - top->y + impl->paint_region.extents.y2 - 1;
-  
+
                   D_DEBUG_AT( GDKDFB_Window, "  -> queue flip %4d,%4d-%4dx%4d (%ld boxes)\n",
                               DFB_RECTANGLE_VALS_FROM_REGION( &reg ), impl->paint_region.numRects );
-  
+
                   dfb_updates_add( &wimpl->flips, &reg );
                 }
             }
-  
+
           temp_region_reset( &impl->paint_region );
         }
     }
@@ -3259,7 +3217,7 @@ gdk_window_set_opacity (GdkWindow *window,
 {
   GdkDisplay *display;
   guint8 cardinal;
-  
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
@@ -3281,6 +3239,33 @@ _gdk_windowing_window_set_composited (GdkWindow *window,
 {
 }
 
+static void
+gdk_window_impl_iface_init (GdkWindowImplIface *iface)
+{
+  iface->show = gdk_directfb_window_show;
+  iface->hide = gdk_directfb_window_hide;
+  iface->withdraw = gdk_directfb_window_withdraw;
+  iface->raise = gdk_window_directfb_raise;
+  iface->lower = gdk_window_directfb_lower;
+  iface->move_resize = gdk_directfb_window_move_resize;
+  iface->move_region = _gdk_directfb_window_move_region;
+  iface->scroll = _gdk_directfb_window_scroll;
+  iface->clear_area = gdk_directfb_window_clear_area;
+  iface->set_background = gdk_directfb_window_set_background;
+  iface->set_back_pixmap = gdk_directfb_window_set_back_pixmap;
+  iface->get_events = gdk_directfb_window_get_events;
+  iface->set_events = gdk_directfb_window_set_events;
+  iface->reparent = gdk_directfb_window_reparent;
+  iface->set_cursor = gdk_directfb_window_set_cursor;
+  iface->get_geometry = gdk_directfb_window_get_geometry;
+  iface->get_origin = gdk_directfb_window_get_origin;
+  iface->get_offsets = _gdk_directfb_window_get_offsets;
+  iface->shape_combine_mask = gdk_directfb_window_shape_combine_mask;
+  iface->shape_combine_region = gdk_directfb_window_shape_combine_region;
+  iface->set_child_shapes = gdk_directfb_window_set_child_shapes;
+  iface->merge_child_shapes = gdk_directfb_window_merge_child_shapes;
+  iface->set_static_gravities = gdk_directfb_window_set_static_gravities;
+}
 
 #define __GDK_WINDOW_X11_C__
 #include "gdkaliasdef.c"
