@@ -21,11 +21,15 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GTK_VBUTTON_BOX_H__
-#define __GTK_VBUTTON_BOX_H__
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
+#ifndef __GTK_VBBOX_H__
+#define __GTK_VBBOX_H__
 
 
 #include <gtk/gtkbbox.h>
@@ -73,4 +77,4 @@ void gtk_vbutton_box_set_layout_default (GtkButtonBoxStyle layout);
 G_END_DECLS
 
 
-#endif /* __GTK_VBUTTON_BOX_H__ */
+#endif /* __GTK_VBBOX_H__ */
